from django.db.models.deletion import ProtectedError
from django.utils.cache import patch_cache_control
from eremaea import models, serializers
from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.parsers import FileUploadParser
from rest_framework.response import Response

class RetentionPolicyViewSet(viewsets.ModelViewSet):
	queryset = models.RetentionPolicy.objects.all()
	serializer_class = serializers.RetentionPolicySerializer
	lookup_field = 'name'

	def destroy(self, request, name):
		try:
			return super(RetentionPolicyViewSet, self).destroy(request, name)
		except ProtectedError as e:
			return Response(status=status.HTTP_400_BAD_REQUEST)
	@action(methods=['post'], detail=True)
	def purge(self, request, name):
		retention_policy = models.RetentionPolicy.objects.get(name = name)
		retention_policy.purge()
		return Response(status=status.HTTP_201_CREATED)

class CollectionViewSet(viewsets.ModelViewSet):
	queryset = models.Collection.objects.select_related("default_retention_policy")
	serializer_class = serializers.CollectionSerializer
	lookup_field = 'name'

	def retrieve(self, request, name=None):
		response = super(CollectionViewSet, self).retrieve(request, name)
		link = []
		if 'begin' in response.data:
			link.append("{0}; rel=begin".format(response.data['begin']))
		if 'end' in response.data:
			link.append("{0}; rel=end".format(response.data['end']))
		if link:
			response['Link'] = ", ".join(link)
		patch_cache_control(response, max_age=0, must_revalidate=True)
		return response

class SnapshotViewSet(viewsets.ModelViewSet):
	queryset = models.Snapshot.objects.select_related("collection", "retention_policy")
	serializer_class = serializers.SnapshotSerializer
	parser_classes = (FileUploadParser,)

	def create(self, request):
		if 'collection' not in request.query_params:
			return Response(status=status.HTTP_400_BAD_REQUEST, data={'detail':'collection is not specified'})
		if 'file' not in request.data:
			return Response(status=status.HTTP_400_BAD_REQUEST, data={'detail':'file is not supplied'})
		kwargs = {}
		kwargs['file'] = request.data['file']
		kwargs['collection'] = models.Collection.objects.get(name = request.query_params['collection'])
		if 'retention_policy' in request.query_params:
			kwargs['retention_policy'] = models.RetentionPolicy.objects.get(name = request.query_params['retention_policy'])
		models.Snapshot.objects.create(**kwargs)
		return Response(status=status.HTTP_201_CREATED)
	def retrieve(self, request, pk=None):
		response = super(SnapshotViewSet, self).retrieve(request, pk)
		link = []
		if response.data['next'] is not None:
			link.append("{0}; rel=next".format(response.data['next']))
		if response.data['prev'] is not None:
			link.append("{0}; rel=prev".format(response.data['prev']))
		link.append("{0}; rel=alternate".format(response.data['file']))
		if link:
			response['Link'] = ", ".join(link)
		return response



import requests

class Client(object):
	def __init__(self, api, token = None):
		self.api = api.rstrip('/')
		self._session = requests.Session()

		if token is not None:
			self._session.headers.update({'Authorization': 'Token {}'.format(token)})

	def upload(self, file, collection, retention_policy = None):
		url = self.api + '/snapshots/'
		headers = {
			'Content-Disposition': 'attachment; filename=\"{}\"'.format(file.name),
			'Content-Type': file.mimetype
		}
		params = {'collection': collection}
		if retention_policy:
			params['retention_policy'] = retention_policy
		r = self._session.post(url, params=params, headers=headers, data=file.content)
		if r.status_code == 201:
			return True
		r.raise_for_status()

	def purge(self, retention_policy):
		url = self.api + '/retention_policies/' + retention_policy + "/purge/"
		r = self._session.post(url)
		if r.status_code == 201:
			return True
		r.raise_for_status()

	def retention_policies(self):
		url = self.api + '/retention_policies/'
		r = self._session.get(url)
		if r.status_code == 200:
			return [x["name"] for x in r.json()]
		r.raise_for_status()



from django.db import models, transaction
from django.db.models import F
from django.utils import timezone
from os import path
from eremaea.conf import settings
import magic
import mimetypes

def guess_extension(instance, filename):
	content_type = getattr(instance.file.file, 'content_type', None)
	_, ext = path.splitext(filename)

	if content_type is not None:
		valid_exts = mimetypes.guess_all_extensions(content_type)
		if ext in valid_exts:
			return ext
		elif len(valid_exts):
			return valid_exts[0]

	if not ext:
		content_type = magic.from_buffer(next(instance.file.file.chunks()), mime=True)
		return mimetypes.guess_extension(content_type)

	return ext

def snapshot_upload_to(instance, filename):
	prefix = settings.PATH
	collection = instance.collection.name
	date = instance.date.strftime("%Y-%m-%d-%H-%M-%S")
	ext = guess_extension(instance, filename)
	newfilename = "{0}-{1}{2}".format(collection, date, ext)
	return path.join(prefix, collection, newfilename)

class SnapshotQuerySet(models.query.QuerySet):
	# Django storage API doesn't support transactions
	# We can't support bulk delete
	def delete(self):
		for x in self:
			x.delete()
	delete.queryset_only = False

class ExpiredSnapshotManager(models.Manager):
	def get_queryset(self):
		return super(ExpiredSnapshotManager, self).get_queryset(
			).annotate(
				expires=models.ExpressionWrapper(
					F('date') + F('retention_policy__duration'),
					output_field=models.DateTimeField())
			).filter(expires__lt = timezone.now())

class Snapshot(models.Model):
	collection = models.ForeignKey('Collection', on_delete=models.CASCADE, db_index=True)
	date = models.DateTimeField(db_index=True, default=timezone.now)
	file = models.FileField(max_length=256, upload_to=snapshot_upload_to)
	retention_policy = models.ForeignKey('RetentionPolicy', on_delete=models.PROTECT, db_index=True)

	objects = SnapshotQuerySet.as_manager()
	expired_objects = ExpiredSnapshotManager.from_queryset(SnapshotQuerySet)()

	def save(self, *args, **kwargs):
		if not self.retention_policy_id:
			self.retention_policy_id = self.collection.default_retention_policy_id
		return super(Snapshot, self).save(*args, **kwargs)

	@transaction.atomic
	def delete(self, *args, **kwargs):
		storage, path = self.file.storage, self.file.name
		super(Snapshot, self).delete(*args, **kwargs)
		if storage.exists(path):
			storage.delete(path)

	def get_next(self):
		try:
			return self.get_next_by_date(collection = self.collection)
		except Snapshot.DoesNotExist:
			pass
	def get_previous(self):
		try:
			return self.get_previous_by_date(collection = self.collection)
		except Snapshot.DoesNotExist:
			pass

	class Meta:
		ordering = ['-date']
		get_latest_by = 'date'

class Collection(models.Model):
	name = models.CharField(max_length=256, blank=False, unique=True, db_index=True)
	default_retention_policy = models.ForeignKey('RetentionPolicy', on_delete=models.PROTECT)

	def get_latest(self):
		try:
			return Snapshot.objects.filter(collection = self).latest()
		except Snapshot.DoesNotExist:
			pass
	def get_earliest(self):
		try:
			return Snapshot.objects.filter(collection = self).earliest()
		except Snapshot.DoesNotExist:
			pass

class RetentionPolicy(models.Model):
	name = models.CharField(max_length=256, blank=False, null=False, unique=True, db_index=True)
	duration = models.DurationField(blank=False, null=False)

	class Meta:
		db_table = 'retention_policy'

	def purge(self):
		return Snapshot.expired_objects.filter(retention_policy = self).delete()



from django.core.files.base import ContentFile
from django.test import TestCase
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APIClient
from mimetypes import guess_all_extensions
from os.path import splitext
from eremaea import models
from datetime import timedelta
from urllib.parse import urlparse

class SnapshotTest(TestCase):
	def setUp(self):
		self.client = APIClient()
		self.retention = models.RetentionPolicy.objects.create(name="daily", duration=timedelta(days=1))
		self.collection = models.Collection.objects.create(name="mycol", default_retention_policy=self.retention)
	def tearDown(self):
		self.collection.delete()
		self.retention.delete()
	
	def assertEqualUrl(self, x, y):
		path_x = urlparse(x).path
		path_y = urlparse(y).path
		return self.assertEqual(path_x, path_y)
	def test_snapshot_create1(self):
		content = b'123'
		url = reverse('snapshot-list')
		response = self.client.post(url, content, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.jpg')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_snapshot_create2(self):
		content = b'123'
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, content, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.jpg')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertEqual(snapshot.retention_policy, self.retention)
		self.assertEqual(snapshot.file.read(), content)
	def test_snapshot_create3(self):
		content = b'123'
		retention_hourly = models.RetentionPolicy.objects.create(name="hourly", duration=timedelta(hours=1))
		url = reverse('snapshot-list') + '?collection=mycol&retention_policy=hourly'
		response = self.client.post(url, content, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.jpg')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertEqual(snapshot.retention_policy, retention_hourly)
		self.assertEqual(snapshot.file.read(), content)
	def test_snapshot_create4(self):
		content = b''
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, content, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.jpg')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_snapshot_create5(self):
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, {}, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.png')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertIn(splitext(snapshot.file.name)[1], guess_all_extensions('image/jpeg'))
	def test_snapshot_create6(self):
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, {}, content_type='image/jpeg', HTTP_CONTENT_DISPOSITION='attachment; filename=upload.jpg')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertEqual(splitext(snapshot.file.name)[1], ".jpg")
	def test_snapshot_create7(self):
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, {}, HTTP_CONTENT_DISPOSITION='attachment; filename=upload.png')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertEqual(splitext(snapshot.file.name)[1], ".png")
	def test_snapshot_create8(self):
		content = b'\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d\x49\x48\x44\x52\x00\x00\x00\x01\x00\x00\x00\x01\x08\x04\x00\x00\x00\xb5\x1c\x0c\x02\x00\x00\x00\x0b\x49\x44\x41\x54\x78\x9c\x63\x62\x60\x00\x00\x00\x09\x00\x03\x19\x11\xd9\xe4\x00\x00\x00\x00\x49\x45\x4e\x44\xae\x42\x60\x82'
		url = reverse('snapshot-list') + '?collection=mycol'
		response = self.client.post(url, content, content_type='application/x-null', HTTP_CONTENT_DISPOSITION='attachment; filename=upload')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshot = models.Snapshot.objects.all()[0]
		self.assertEqual(splitext(snapshot.file.name)[1], ".png")
	def test_snapshot_get1(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		snapshot = models.Snapshot.objects.create(collection = self.collection, file = file)
		url = reverse('snapshot-detail', args=[snapshot.id])
		response = self.client.get(url)
		self.assertIsNone(response.data['next'])
		self.assertIsNone(response.data['prev'])
	def test_snapshot_get2(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		snapshot1 = models.Snapshot.objects.create(collection = self.collection, file = file)
		snapshot2 = models.Snapshot.objects.create(collection = self.collection, file = file)
		snapshot3 = models.Snapshot.objects.create(collection = self.collection, file = file)
		url = reverse('snapshot-detail', args=[snapshot1.id])
		response = self.client.get(url)
		self.assertEqualUrl(response.data['next'], reverse("snapshot-detail", args=(snapshot2.id,)))
		self.assertIsNone(response.data['prev'])
		url = reverse('snapshot-detail', args=[snapshot2.id])
		response = self.client.get(url)
		self.assertEqualUrl(response.data['next'], reverse("snapshot-detail", args=(snapshot3.id,)))
		self.assertEqualUrl(response.data['prev'], reverse("snapshot-detail", args=(snapshot1.id,)))
		url = reverse('snapshot-detail', args=[snapshot3.id])
		response = self.client.get(url)
		self.assertIsNone(response.data['next'])
		self.assertEqualUrl(response.data['prev'], reverse("snapshot-detail", args=(snapshot2.id,)))
	def test_snapshot_get3(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		retention_hourly = models.RetentionPolicy.objects.create(name="hourly", duration=timedelta(hours=1))
		collection1 = models.Collection.objects.create(name="collection1", default_retention_policy=retention_hourly)
		collection2 = models.Collection.objects.create(name="collection2", default_retention_policy=retention_hourly)
		snapshots1 = []
		snapshots2 = []
		for i in range(0,3):
			snapshots1.append(models.Snapshot.objects.create(collection = collection1, file = file))
			snapshots2.append(models.Snapshot.objects.create(collection = collection2, file = file))
		# first collection
		self.assertEqual(snapshots1[0].get_next(), snapshots1[1])
		self.assertEqual(snapshots1[1].get_next(), snapshots1[2])
		self.assertEqual(snapshots1[1].get_previous(), snapshots1[0])
		self.assertEqual(snapshots1[2].get_previous(), snapshots1[1])
		self.assertEqual(collection1.get_earliest(), snapshots1[0])
		self.assertEqual(collection1.get_latest(), snapshots1[2])
		# second collection
		self.assertEqual(snapshots2[0].get_next(), snapshots2[1])
		self.assertEqual(snapshots2[1].get_next(), snapshots2[2])
		self.assertEqual(snapshots2[1].get_previous(), snapshots2[0])
		self.assertEqual(snapshots2[2].get_previous(), snapshots2[1])
		self.assertEqual(collection2.get_earliest(), snapshots2[0])
		self.assertEqual(collection2.get_latest(), snapshots2[2])
	def test_snapshot_get4(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		snapshot = models.Snapshot.objects.create(collection = self.collection, file = file)
		url = reverse('snapshot-detail', args=[snapshot.id])
		response = self.client.get(url)
		link_hdr = response['Link']
		self.assertEqual(link_hdr, '{}; rel=alternate'.format(response.data['file']))
	def test_snapshot_head1(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		snapshot = models.Snapshot.objects.create(collection = self.collection, file = file)
		url = reverse('snapshot-detail', args=[snapshot.id])
		response = self.client.head(url)
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		self.assertIsNone(response.data['next'])
		self.assertIsNone(response.data['prev'])
	def test_snapshot_delete1(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		snapshot = models.Snapshot.objects.create(collection = self.collection, file = file)
		storage = snapshot.file.storage
		filepath = snapshot.file.name
		url = reverse('snapshot-detail', args=[snapshot.id])
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
		self.assertFalse(storage.exists(filepath))



from collections import namedtuple
from urllib.parse import urlparse
import os.path
import mimetypes
import requests
from requests_toolbelt.auth.guess import GuessAuth


File = namedtuple("File", ("name", "mimetype", "content"))

class Stream:
	def __init__(self, url):
		self._url = url

	def __iter__(self):
		return self

	def __next__(self):
		raise NotImplementedError

class LocalFileStream(Stream):
	def __init__(self, url):
		super().__init__(url)

	def __next__(self):
		with open(self._url, "rb") as f:
			(mimetype, _encoding) = mimetypes.guess_type(self._url)
			name = os.path.basename(self._url)
			return File(name, mimetype, f.read())

class HTTPFileStream(Stream):
	def __init__(self, url):
		super().__init__(url)

		parsed_url = urlparse(url)

		self._name = [x for x in parsed_url.path.rsplit("/") if x][-1]
		self._session = requests.Session()
		self._session.allow_redirects = True

		if parsed_url.username is not None:
			login = parsed_url.username
			passwd = parsed_url.password if parsed_url.password is not None else ""
			self._session.auth = GuessAuth(login, passwd)

	def __next__(self):
		r = self._session.get(self._url)
		r.raise_for_status()

		mimetype = r.headers.get('content-type')
		if mimetype is None:
			(mimetype, _encoding) = mimetypes.guess_type(self._name)

		return File(self._name, mimetype, r.content)


def create_stream(url):
	streams = {
		'': LocalFileStream,
		'http': HTTPFileStream,
		'https': HTTPFileStream,
	}

	parsed_url = urlparse(url)

	stream_class = streams.get(parsed_url.scheme)

	if stream_class is not None:
		return stream_class(url)

	return None



from eremaea import models
from rest_framework import serializers

class RetentionPolicySerializer(serializers.HyperlinkedModelSerializer):
	url = serializers.HyperlinkedIdentityField(lookup_field='name', view_name='retention_policy-detail')

	class Meta:
		model = models.RetentionPolicy
		fields = ('url', 'name', 'duration')

class SnapshotSerializer(serializers.HyperlinkedModelSerializer):
	collection = serializers.HyperlinkedRelatedField(view_name='collection-detail', read_only=True, lookup_field='name')
	retention_policy = serializers.HyperlinkedRelatedField(view_name='retention_policy-detail', queryset=models.RetentionPolicy.objects.all(), lookup_field='name')
	next = serializers.HyperlinkedRelatedField(view_name='snapshot-detail', read_only=True, source="get_next")
	prev = serializers.HyperlinkedRelatedField(view_name='snapshot-detail', read_only=True, source="get_previous")

	class Meta:
		model = models.Snapshot
		fields = ('url', 'collection', 'file', 'date', 'retention_policy', 'next', 'prev')

class CollectionSerializer(serializers.HyperlinkedModelSerializer):
	url = serializers.HyperlinkedIdentityField(lookup_field='name', view_name='collection-detail')
	default_retention_policy = serializers.HyperlinkedRelatedField(view_name='retention_policy-detail', queryset=models.RetentionPolicy.objects.all(), lookup_field='name')
	begin = serializers.HyperlinkedRelatedField(view_name='snapshot-detail', read_only=True, source="get_earliest")
	end = serializers.HyperlinkedRelatedField(view_name='snapshot-detail', read_only=True, source="get_latest")

	class Meta:
		model = models.Collection
		fields = ('url', 'name', 'default_retention_policy', 'begin', 'end')



import django.conf

DEFAULTS = {
	'PATH': 'eremaea',
}

class Settings(object):
	def __init__(self, defaults=None):
		self.defaults = defaults or DEFAULTS
	def __getattr__(self, attr):
		s = getattr(django.conf.settings, 'EREMAEA', {})
		return s.get(attr, self.defaults[attr])

settings = Settings(DEFAULTS)

__all__ = ["settings"]



try:
	from django.urls import reverse
except ImportError:
	from django.core.urlresolvers import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient
from eremaea import models
from datetime import timedelta
from urllib.parse import urlparse

class CollectionTest(TestCase):
	def setUp(self):
		self.client = APIClient()
		self.retention = models.RetentionPolicy.objects.create(name="daily", duration=timedelta(days=1))
	def tearDown(self):
		pass
	
	def assertEqualUrl(self, x, y):
		path_x = urlparse(x).path
		path_y = urlparse(y).path
		return self.assertEqual(path_x, path_y)
	def test_collection_create1(self):
		url = reverse('collection-list')
		retention_url = reverse('retention_policy-detail', args=('daily',))
		response = self.client.post(url, {'name': 'name1', 'default_retention_policy': retention_url}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		response = self.client.post(url, {'name': 'name1', 'default_retention_policy': retention_url}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		group1 = models.Collection.objects.get(name="name1")
		self.assertEqual(group1.default_retention_policy, self.retention)
		url = reverse('collection-detail', args=['name1'])
		response = self.client.get(url, format='json')
		self.assertEqual(response.status_code, status.HTTP_200_OK)
	def test_collection_get1(self):
		collection = models.Collection.objects.create(name="collection", default_retention_policy=self.retention)
		snapshots = []
		for i in range(0,3):
			snapshots.append(models.Snapshot.objects.create(collection = collection))
		url = reverse('collection-detail', args=('collection',))
		response = self.client.get(url)
		self.assertEqualUrl(response.data['begin'], reverse("snapshot-detail", args=(snapshots[0].id,)))
		self.assertEqualUrl(response.data['end'], reverse("snapshot-detail", args=(snapshots[2].id,)))
	def test_collection_head1(self):
		collection = models.Collection.objects.create(name="collection", default_retention_policy=self.retention)
		snapshots = []
		for i in range(0,3):
			snapshots.append(models.Snapshot.objects.create(collection = collection))
		url = reverse('collection-detail', args=('collection',))
		response = self.client.head(url)
		self.assertEqual(response.status_code, status.HTTP_200_OK)
		self.assertEqualUrl(response.data['begin'], reverse("snapshot-detail", args=(snapshots[0].id,)))
		self.assertEqualUrl(response.data['end'], reverse("snapshot-detail", args=(snapshots[2].id,)))
	def test_collection_delete1(self):
		group1 = models.Collection.objects.create(name="delete1", default_retention_policy=self.retention)
		url = reverse('collection-detail', args=['delete1'])
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
		self.assertRaises(models.Collection.DoesNotExist, models.Collection.objects.get, name="delete1")




import cmdln
import time
import sys
from eremaea.ctl.file import create_stream
from eremaea.ctl.client import Client
from django.utils.dateparse import parse_duration

class CommandLine(cmdln.Cmdln, object):
	name = "eremaeactl"

	def get_optparser(self):
		parser = cmdln.Cmdln.get_optparser(self)
		parser.add_option("-A", dest="api_endpoint", help="HTTP REST API endpoint URL", default="http://127.0.0.1/eremaea")
		parser.add_option("-T", dest="api_token", help="HTTP Token Authorization")
		return parser
	def precmd(self, argv):
		kwargs = {}
		if self.options.api_token:
			kwargs['token'] = self.options.api_token
		self.client = Client(self.options.api_endpoint, **kwargs)
		return super(CommandLine, self).precmd(argv)
	def postcmd(self, argv):
		del self.client

	@cmdln.alias("up")
	@cmdln.option("-q", "--quite", action="store_true", help="be quite")
	@cmdln.option("-r", dest="retention_policy", help="specify retention policy (optional)")
	def do_upload(self, subcmd, opts, file, collection):
		"""${cmd_name}: upload file to storage

		${cmd_usage}
		${cmd_option_list}
		"""
		self.client.upload(next(create_stream(file)), collection, opts.retention_policy)
	@cmdln.option("--all", dest="all", action="store_true", help="purge all retention policies")
	def do_purge(self, subcmd, opts, *retention_policies):
		"""${cmd_name}: purge retention policies

		${cmd_usage}
		${cmd_option_list}
		"""
		if opts.all:
			retention_policies = self.client.retention_policies()
		if not retention_policies and not opts.all:
			CommandLine.do_purge.optparser.print_help()
		for x in retention_policies:
			self.client.purge(x)
	@cmdln.option("-q", "--quite", action="store_true", help="be quite")
	@cmdln.option("-i", "--interval", help="pull interval", default="1:00")
	@cmdln.option("-r", dest="retention_policy", help="specify retention policy (optional)")
	def do_pull(self, subcmd, opts, file, collection):
		"""${cmd_name}: pull image

		${cmd_usage}
		${cmd_option_list}
		"""
		duration = parse_duration(opts.interval)
		stream = create_stream(file)
		while True:
			try:
				self.client.upload(next(stream), collection, opts.retention_policy)
			except Exception as e:
				if not opts.quite:
					sys.stderr.write(str(e) + "\n")
					sys.stderr.flush()
			time.sleep(duration.total_seconds())

def execute_from_commandline(argv=None):
	cmd = CommandLine()
	cmd.main(argv)



from django.core.files.base import ContentFile
try:
	from django.urls import reverse
except ImportError:
	from django.core.urlresolvers import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient
from eremaea import models
from datetime import datetime,timedelta

class RetentionPolicyTest(TestCase):
	def setUp(self):
		self.client = APIClient()
	
	def test_retention_create1(self):
		url = reverse('retention_policy-list')
		response = self.client.post(url, {'name': 'daily', 'duration': '01 00'}, format='json')
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		response = self.client.post(url, {'name': 'daily', 'duration': '02 00'}, format='json')
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
		group1 = models.RetentionPolicy.objects.get(name="daily")
		self.assertEqual(group1.duration, timedelta(days=1))
		url = reverse('retention_policy-detail', args=['daily'])
		response = self.client.get(url, format='json')
		self.assertEqual(response.data['duration'], '1 00:00:00')
		self.assertEqual(response.status_code, status.HTTP_200_OK)
	def test_retention_delete1(self):
		group1 = models.RetentionPolicy.objects.create(id=123, name="test", duration=timedelta(days=1))
		url = reverse('retention_policy-detail', args=['test'])
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
		self.assertRaises(models.RetentionPolicy.DoesNotExist, models.RetentionPolicy.objects.get, name="test")
	def test_retention_delete2(self):
		# Retention deletion is protected by collection
		retention = models.RetentionPolicy.objects.create(name='daily', duration=timedelta(days=1))
		collection = models.Collection.objects.create(name='test', default_retention_policy=retention)
		url = reverse('retention_policy-detail', args=['daily'])
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_retention_delete3(self):
		# Retention deletion is protected by snapshot
		retention_daily  = models.RetentionPolicy.objects.create(name='daily',  duration=timedelta(days=1))
		retention_hourly = models.RetentionPolicy.objects.create(name='hourly', duration=timedelta(hours=1))
		collection = models.Collection.objects.create(name='test', default_retention_policy=retention_daily)
		snapshot = models.Snapshot.objects.create(collection = collection, retention_policy=retention_hourly)
		url = reverse('retention_policy-detail', args=['hourly'])
		response = self.client.delete(url)
		self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
	def test_retention_purge1(self):
		file = ContentFile(b"123")
		file.name = "file.jpg"
		retention_daily  = models.RetentionPolicy.objects.create(name='daily',  duration=timedelta(days=1))
		retention_hourly = models.RetentionPolicy.objects.create(name='hourly', duration=timedelta(hours=1))
		collection = models.Collection.objects.create(name='test', default_retention_policy=retention_hourly)
		dates = [datetime.now(), datetime.now() - timedelta(minutes=30), datetime.now() - timedelta(minutes=90)]
		snapshots = [models.Snapshot.objects.create(collection = collection, date = x, file = file) for x in dates]
		snapshots.append(models.Snapshot.objects.create(collection = collection, date = dates[-1], retention_policy = retention_daily, file = file))
		storage2, filepath2 = snapshots[2].file.storage, snapshots[2].file.name
		url = reverse('retention_policy-purge', args=['hourly'])
		response = self.client.post(url)
		self.assertEqual(response.status_code, status.HTTP_201_CREATED)
		snapshots2 = models.Snapshot.objects.all()
		self.assertEqual(len(snapshots2), 3)
		self.assertEqual(snapshots2[0], snapshots[0])
		self.assertEqual(snapshots2[1], snapshots[1])
		self.assertEqual(snapshots2[2], snapshots[3])
		self.assertFalse(storage2.exists(filepath2))



import django
from eremaea import views
from rest_framework.routers import DefaultRouter

router = DefaultRouter()
router.register(r'collections', views.CollectionViewSet)
router.register(r'snapshots', views.SnapshotViewSet)
router.register(r'retention_policies', views.RetentionPolicyViewSet, basename='retention_policy')

if django.VERSION[0] > 1:
	from django.urls import include, re_path
else:
	from django.conf.urls import include, url as re_path

urlpatterns = [
	re_path(r'^', include(router.urls)),
]



# Generated by Django 5.0.8 on 2024-09-06 11:28

import django.db.models.deletion
import django.utils.timezone
import eremaea.models
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [('eremaea', '0001_initial'), ('eremaea', '0002_drop_index_together')]

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RetentionPolicy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=256, unique=True)),
                ('duration', models.DurationField()),
            ],
            options={
                'db_table': 'retention_policy',
            },
        ),
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=256, unique=True)),
                ('default_retention_policy', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='eremaea.retentionpolicy')),
            ],
        ),
        migrations.CreateModel(
            name='Snapshot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('file', models.FileField(max_length=256, upload_to=eremaea.models.snapshot_upload_to)),
                ('collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eremaea.collection')),
                ('retention_policy', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='eremaea.retentionpolicy')),
            ],
            options={
                'ordering': ['-date'],
                'get_latest_by': 'date',
            },
        ),
    ]



from django.test import TestCase, override_settings
from eremaea import conf

class ConfTest(TestCase):
	def setUp(self):
		pass
	def tearDown(self):
		pass
	def test_conf1(self):
		self.assertEqual(conf.settings.PATH, "eremaea")
	@override_settings(EREMAEA={'PATH':'custom_path'})
	def test_conf2(self):
		self.assertEqual(conf.settings.PATH, "custom_path")
